#
# Copyright (C) 2014 - present Instructure, Inc.
#
# This file is part of Canvas.
#
# Canvas is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# Canvas is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

define [
  '../mockFilesENV.coffee'
  'react'
  'react-dom'
  'react-addons-test-utils'
  'jquery'
  'jsx/shared/PublishCloud'
  'compiled/models/FilesystemObject'
], (mockFilesEnv, React, ReactDOM, {Simulate}, $, PublishCloud, FilesystemObject) ->

  # Integration Tests
  QUnit.module 'PublishCloud',
    setup: ->
      @model = new FilesystemObject(locked: true, hidden: false, id: 42)
      @model.url = -> "/api/v1/folders/#{@id}"
      props =
        model: @model
        userCanManageFilesForContext: true

      @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])

    teardown: ->
      ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)

  test "model change event updates components state", ->
    equal @publishCloud.state.published, false, "published starts off as false"
    @model.set('locked', false)
    equal @publishCloud.state.published, true, "changing models locked changes it to true"

  test "clicking a published cloud opens restricted dialog", ->
    @stub(ReactDOM, 'render')
    Simulate.click(@publishCloud.refs.publishCloud.getDOMNode())

    ok ReactDOM.render.calledOnce, 'renders a component inside the dialog'

  QUnit.module 'PublishCloud Student View',
    setup: ->
      @model = new FilesystemObject(locked: false, hidden: true, lock_at: '2014-02-01', unlock_at: '2014-01-01', id: 42)
      @model.url = -> "/api/v1/folders/#{@id}"
      props =
        model: @model
        userCanManageFilesForContext: false

      @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])

    teardown: ->
      ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)

  test 'should display a non clickable restricted dates icon', ->
    equal @publishCloud.refs.publishCloud.props.onClick, undefined, 'does not have a click event'
    equal @publishCloud.refs.publishCloud.props.title, "Available after Jan 1, 2014 at 12am until Feb 1, 2014 at 12am", "has a available from hoverover"

  # Unit Tests

  QUnit.module 'PublishCloud#togglePublishedState',
    setup: ->
      props =
        model: new FilesystemObject(hidden: false, id: 42)
        userCanManageFilesForContext: true

      @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])

    teardown: ->
      ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)

  test "when published is true, toggles it to false", ->
    @publishCloud.setState published: true
    @publishCloud.togglePublishedState()
    equal @publishCloud.state.published, false, "published state should be false"

  test "when published is false, toggles publish to true and clears restricted state", ->
    @publishCloud.setState published: false, restricted: true
    @publishCloud.togglePublishedState()
    equal @publishCloud.state.published, true, "published state should be true"
    equal @publishCloud.state.restricted, false, "published state should be true"

  test "when published is false, toggles publish to true and sets hidden to false", ->
    @publishCloud.setState published: false, restricted: true
    @publishCloud.togglePublishedState()
    equal @publishCloud.state.published, true, "published state should be true"
    equal @publishCloud.state.hidden, false, "hidden is false"

  QUnit.module 'PublishCloud#getInitialState'

  test "sets published initial state based on params model hidden property", ->
    model = new FilesystemObject(locked: false, id: 42)
    props =
      model: model
      userCanManageFilesForContext: true

    @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])
    equal @publishCloud.state.published, !model.get('locked'), "not locked is published"
    equal @publishCloud.state.restricted, false, "restricted should be false"
    equal @publishCloud.state.hidden, false, "hidden should be false"
    ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)

  test "restricted is true when lock_at/unlock_at is set", ->
    model = new FilesystemObject(hidden: false, lock_at: '123', unlock_at: '123', id: 42)
    props = model: model

    @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])

    equal @publishCloud.state.restricted, true, "restricted is true when lock_at/ulock_at is set"
    ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)

  QUnit.module 'PublishCloud#extractStateFromModel'

  test "returns object that can be used to set state", ->
    model = new FilesystemObject(locked: true, hidden: true, lock_at: '123', unlock_at: '123', id: 42)
    props = model: model
    @publishCloud = ReactDOM.render(React.createElement(PublishCloud, props), $('#fixtures')[0])

    newModel = new FilesystemObject(locked: false, hidden: true, lock_at: null, unlock_at: null)
    deepEqual @publishCloud.extractStateFromModel(newModel), {hidden: true, published: true, restricted: false}, "returns object to set state with"
    ReactDOM.unmountComponentAtNode(@publishCloud.getDOMNode().parentNode)
